/*
 *  This file contains Good Sample Code subject to the Good Dynamics SDK Terms and Conditions.
 *  (c) 2013 Good Technology Corporation. All rights reserved.
 */

#import <GD/GDFileSystem.h>
#import "FeedDownloadGDHttp.h"

#define kAuthDialog       1
#define kSSLDialog        2
#define kErrorAlertDialog 101

const char* kAuthBasicStr     = "Basic";
const char* kAuthDigestStr    = "Digest";
const char* kAuthNTLMStr      = "NTLM";
const char* kAuthNegotiateStr = "Negotiate";

@interface FeedDownloadHttpGDHttp ()

- (void)requestData:(NSString*)url useridOrNil:(NSString*)userId passwordOrNil:(NSString*)password relaxSSL:(BOOL)relaxSSL;
- (void)displaySSLQueryDialog;
- (void)displayAuthQueryDialog;
- (void)handleAuthQuery:(NSInteger)buttonIndex;
- (void)handleSSLQuery:(NSInteger)buttonIndex;

@end

@implementation FeedDownloadHttpGDHttp

@synthesize authMethod;

- (id)init
{
    self = [super init];
    if (self)
    {
        httpRequest = [[GDHttpRequest alloc] init];
        // Optionally, to use an HTTP Proxy, uncomment the code below...
        // [httpRequest enableHttpProxy:"host" withPort:3128 withUser:"username" withPass:"password" withAuth:"DIGEST"];
        // or, if no proxy authentication is required...
        // [httpRequest enableHttpProxy:"host" withPort:3128];
        
        httpRequest.delegate = self;
    }
    return self;
}

#if !(__has_feature(objc_arc))
- (void)dealloc
{
    [httpRequest release];
    [authMethod release];
    [super dealloc];
}
#endif

#pragma mark - Request Control

/* Requests the data from a specific URL. This can trigger SSL relaxation or authentication dialogs
 */
- (void)requestData:(NSString*)url
{
    relaxCurrentSSL = NO;
    currentURL = url;
    self.authMethod = nil;
    
    [self requestData:url useridOrNil:nil passwordOrNil:nil relaxSSL:relaxCurrentSSL];
}

/* Internal request method that can take auth parameters and relax the SSL if required
 */
- (void)requestData:(NSString*)url useridOrNil:(NSString*)userId passwordOrNil:(NSString*)password relaxSSL:(BOOL)relaxSSL
{
    
    // check SSL relaxation
    if (relaxSSL)
    {
        [httpRequest disablePeerVerification];
    }
    
    // set credentials
    if (userId && password && authMethod)
    {
        // open the request
        [httpRequest open:"GET" withUrl:[url UTF8String] withAsync:YES
                 withUser:[userId UTF8String] withPass:[password UTF8String] withAuth:[authMethod UTF8String]];
    }
    else
    {
        // open the request
        [httpRequest open:"GET" withUrl:[url UTF8String] withAsync:YES];
    }
}

- (void)abortRequest
{
    [httpRequest abort];
    [UIApplication sharedApplication].networkActivityIndicatorVisible = NO;
}

#pragma mark - GDHttpRequest delegate

/*
 * This is the GDHttpRequest status delegate. When the request changes state it calls back to this method
 * in which we query the request to get the state. Since we requested an asynchronous open we send the request
 * when we enter the opened state. When we receive the done state call we extract the buffer and parse the data.
 */
- (void)onStatusChange:(GDHttpRequest*)request
{
    int state = (int)[request getState];
    bool errorDialog = false;
    
    switch (state)
    {
        case GDHttpRequest_OPENED:
        {
            // request opened so now send
            [request send];
            break;
        }
            
        case GDHttpRequest_HEADERS_RECEIVED:
        {
            // do nothing in this example
            break;
        }
            
        case GDHttpRequest_DONE:
        {
            // check status
            int status = [request getStatus];
            
            // check to see if we have a SSL certificate issue
            // and if so give the user the chance to relax the
            // acceptance criteria
            if (status == 0)
            {
                const char* err = [request getStatusText];
                char* tmp = strcasestr(err, "SSL");
                if (tmp)
                {
                    [self displaySSLQueryDialog];
                }
                else
                {
                    errorDialog = true;
                }
            }
            else if (status == 200)
            {
                // the request has finished so see if we have any data
                GDDirectByteBuffer* buf  = [request getReceiveBuffer];
                int len  = [buf bytesUnread];
                char raw[len];
                [buf read:raw toMaxLength:len];
#if !(__has_feature(objc_arc))
                NSData* data  = [[[NSData alloc] initWithBytes:raw length:len] autorelease];
#else
                NSData* data  = [[NSData alloc] initWithBytes:raw length:len];
#endif
                // if we have data then parse via the XML parser so call the delegate method
                if ([data length] > 0)
                {
                    [self.delegate downloadDone:data];
                }
            }
            else if ((status == 401) || (status  == 407))
            {
                // assuming NTLM authentication, extract headers to determine what the server supports
                const char* headerData = [httpRequest getResponseHeader:"WWW-Authenticate"];
                if (!strncasecmp(kAuthNegotiateStr, headerData, strlen(kAuthNegotiateStr)))
                {
                    self.authMethod = [NSString stringWithUTF8String:kAuthNegotiateStr];
                    [self displayAuthQueryDialog];
                }
                else if (!strncasecmp(kAuthNTLMStr, headerData, strlen(kAuthNTLMStr)))
                {
                    self.authMethod = [NSString stringWithUTF8String:kAuthNTLMStr];
                    [self displayAuthQueryDialog];
                }
                else if (!strncasecmp(kAuthDigestStr, headerData, strlen(kAuthDigestStr)))
                {
                    self.authMethod = [NSString stringWithUTF8String:kAuthDigestStr];
                    [self displayAuthQueryDialog];
                }
                else if (!strncasecmp(kAuthBasicStr, headerData, strlen(kAuthBasicStr)))
                {
                    self.authMethod = [NSString stringWithUTF8String:kAuthBasicStr];
                    [self displayAuthQueryDialog];
                }
                else
                {
                    // only the above methods currently supported in this application
                    errorDialog = true;
                }
            }
            else
            {
                errorDialog = true;
            }
            break;
        }
    }
    
    if (errorDialog)
    {
        const char* err = [request getStatusText];
        if (err)
        {
            NSString* str = [NSString stringWithCString:err encoding:NSUTF8StringEncoding];
            UIAlertView* dialog = [[UIAlertView alloc] initWithTitle:@"Error Fetching File"
                                                             message:str
                                                            delegate:self
                                                   cancelButtonTitle:@"OK" // index 0
                                                   otherButtonTitles:nil];
            dialog.tag = kErrorAlertDialog;
            [dialog show];
#if !(__has_feature(objc_arc))
            [dialog release];
#endif
        }
    }
}

#pragma mark - Invoke SSL Query Dialog

/*
 * This method displays a dialog allowing the user allow relaxation of certificate verification
 */
- (void)displaySSLQueryDialog
{
    // create an alert view to query certificate verification
    UIAlertView* dialog = [[UIAlertView alloc] initWithTitle:@"SSl Certificate Error"
                                                     message:@"Relax certificate verification for this site ?\n"
                                                    delegate:self
                                           cancelButtonTitle:@"No" // index 0
                                           otherButtonTitles:@"Yes", nil];
    dialog.tag = kSSLDialog;
    
    [dialog show];
#if !(__has_feature(objc_arc))
    [dialog release];
#endif
}

#pragma mark - Invoke Auth Query Dialog
/*
 * This method displays a dialog allowing the user to enter a user id and password
 */
- (void)displayAuthQueryDialog
{
    // create an alert view with our user and password fields
    UIAlertView* dialog = [[UIAlertView alloc] initWithTitle:@"Enter ID and Password"
                                                     message:@""
                                                    delegate:self
                                           cancelButtonTitle:@"Cancel" // index 0
                                           otherButtonTitles:@"OK", nil];
    
    dialog.alertViewStyle = UIAlertViewStyleLoginAndPasswordInput;
    dialog.tag = kAuthDialog;
    
    [dialog show];
#if !(__has_feature(objc_arc))
    [dialog release];
#endif
}

#pragma mark - UIAlertViewDelegate

/*
 * Method from UIAlertViewDelegate that is actioned when a button is clicked.
 * Here we are using it to give the user a chance to relax SSL certificate verification
 *
 */
- (void)alertView:(UIAlertView*)alert clickedButtonAtIndex:(NSInteger)buttonIndex
{
    switch (alert.tag)
    {
        case kSSLDialog:
            [self handleSSLQuery:buttonIndex];
            break;
            
        case kAuthDialog:
            userNameField = [alert textFieldAtIndex:0];
            passwordField = [alert textFieldAtIndex:1];
            [self handleAuthQuery:buttonIndex];
            break;
            
        case kErrorAlertDialog:
            [self.delegate downloadDone:nil];
            break;
            
        default:
            NSLog(@"Unknown query dialog");
            break;
    }
}

#pragma mark - Query Handlers

/*
 * This method handles the response from the auth query and reissues a request with the supplied credentials
 * or fails in response to a cancel
 */

- (void)handleAuthQuery:(NSInteger)buttonIndex
{
    if (buttonIndex == 1)
    {
        // set the url (only handle NTLM authentication for this example)
        [self requestData:currentURL useridOrNil:userNameField.text passwordOrNil:passwordField.text relaxSSL:relaxCurrentSSL];
    }
    else
    {
        [self abortRequest];
    }
}

/*
 * This method handles the response from the SSL query and reissues a request with relaxed verification
 * or fails in response to a cancel
 */

- (void)handleSSLQuery:(NSInteger)buttonIndex
{
    if (buttonIndex == 1)
    {
        // set the url
        relaxCurrentSSL = YES;
        [self requestData:currentURL useridOrNil:nil passwordOrNil:nil relaxSSL:relaxCurrentSSL];
    }
}

@end
