/*
 *  This file contains Good Sample Code subject to the Good Dynamics SDK Terms and Conditions.
 *  (c) 2016 BlackBerry Limited. All rights reserved.
 */

#import <GD/GDFileSystem.h>
#import <GD/NSMutableURLRequest+GDNET.h>
#import <GD/GD_C_NETUtility.h>
#import "FeedDownloadNSURLSession.h"

#define kAuthDialog 1
#define kSSLDialog  2
#define kErrorAlertDialog 101

@interface FeedDownloadNSURLSession ()
- (void)requestData:(NSString*)url relaxSSL:(BOOL)relaxSSL;
- (void)displaySSLQueryDialog;
- (void)displayAuthQueryDialog;
- (void)handleAuthQuery:(NSInteger)buttonIndex;
- (void)handleSSLQuery:(NSInteger)buttonIndex;
- (NSString*) decomposeUrl:(NSString *) url;

@property (nonatomic, assign) BOOL isRelaxSSL;
@end

@implementation FeedDownloadNSURLSession

- (id)init
{
    self = [super init];
    if (self)
    {
        dataBuffer = [[NSMutableData alloc] init];
    }
    return self;
}

#if !(__has_feature(objc_arc))
- (void)dealloc
{
    [challenge release];
    [dataBuffer release];
    [super dealloc];
}
#endif

#pragma mark - Request Control

/* Requests the data from a specific URL. This can trigger SSL relaxation or authentication dialogs
 */
- (void)requestData:(NSString*)url
{
    relaxCurrentSSL = NO;
    currentURL = url;
    
    [self requestData:url relaxSSL:relaxCurrentSSL];
}

- (NSURLSessionConfiguration *)getSessionConfiguration {
    NSURLSessionConfiguration *sessionConfiguration = [NSURLSessionConfiguration defaultSessionConfiguration];
    sessionConfiguration.timeoutIntervalForRequest = 60.0f;
    
    return sessionConfiguration;
}

/* Internal request method that can take auth parameters and relax the SSL if required
 */
- (void)requestData:(NSString*)url relaxSSL:(BOOL)relaxSSL
{
    [UIApplication sharedApplication].networkActivityIndicatorVisible = YES;
    
    NSURL* nsUrl = [NSURL URLWithString:url];
    
    // create the url/request
    NSMutableURLRequest* request = [NSMutableURLRequest requestWithURL:nsUrl];
    
    //    Optionally, to use an HTTP Proxy, uncomment the code below...
    //    NSURL* proxyUrl = [NSURL URLWithString:@"http://<hostname>:<port>"];
    //    NSURLCredential* proxyCred = [NSURLCredential credentialWithUser:@"username"
    //                                                            password:@"password"
    //                                                         persistence:NSURLCredentialPersistenceForSession];
    //
    //    NSURLProtectionSpace* proxyProtSpace = [[NSURLProtectionSpace alloc]
    //                                             initWithProxyHost:[proxyUrl host]
    //                                                          port:[[proxyUrl port] integerValue]
    //                                                          type:[proxyUrl scheme]
    //                                                         realm:nil
    //                                          authenticationMethod:NSURLAuthenticationMethodHTTPDigest];
    //    [request setAuthorizationCredentials:proxyCred withProtectionSpace:proxyProtSpace];
    
    self.isRelaxSSL = relaxSSL;
    
    // reset the data buffer for the new request
    [dataBuffer setLength:0];
    
    // open the request
    currentSession = [NSURLSession sessionWithConfiguration:[self getSessionConfiguration]
                                                   delegate:self
                                              delegateQueue:nil];
    NSURLSessionDataTask *dataTask = [currentSession dataTaskWithRequest:request];
    [dataTask resume];
}

- (void)abortRequest
{
    // cancel the session if still active
    [currentSession invalidateAndCancel];
    
    // set to nil
    currentSession = nil;
    
    // switch off the activity indicator here as the normal loading flow will be halted
    [UIApplication sharedApplication].networkActivityIndicatorVisible = NO;
}

- (void)URLSession:(NSURLSession *)session
              task:(NSURLSessionTask *)task
didReceiveChallenge:(NSURLAuthenticationChallenge *)challenge
 completionHandler:(void (^)(NSURLSessionAuthChallengeDisposition disposition, NSURLCredential *credential))completionHandler
{
    NSString* authMethod = [[challenge protectionSpace] authenticationMethod];
    NSLog(@"Auth method in use: %@", authMethod);
    
    if ([authMethod isEqualToString:NSURLAuthenticationMethodHTTPBasic] ||
        [authMethod isEqualToString:NSURLAuthenticationMethodHTTPDigest] ||
        [authMethod isEqualToString:NSURLAuthenticationMethodNTLM] ||
        [authMethod isEqualToString:NSURLAuthenticationMethodNegotiate])
    {
        [self displayAuthQueryDialog];
#if !(__has_feature(objc_arc))
        authChallenge = [challange retain];
#else
        authChallenge = challenge;
#endif
    }
    else if ([authMethod isEqualToString:NSURLAuthenticationMethodClientCertificate])
    {
        // do nothing - GD will automatically supply an appropriate client certificate
    }
    else if ([authMethod isEqualToString:NSURLAuthenticationMethodServerTrust])
    {
        // check SSL relaxation
        if (self.isRelaxSSL &&
            [authMethod isEqualToString:NSURLAuthenticationMethodServerTrust]) {
            NSURLCredential *credential = [NSURLCredential credentialForTrust:challenge.protectionSpace.serverTrust];
            
            completionHandler(NSURLSessionAuthChallengeUseCredential, credential);
        }
    }
    else
    {
        // this application doesn't support the specified method
        [delegate downloadDone:nil];
    }
    
    // reset the buffer
    [dataBuffer setLength:0];
}

- (NSString*) decomposeUrl:(NSString *) url
{
    NSString* host;
    
    NSRange doubleSlash = [url rangeOfString:@"//"];
    NSInteger hostPortStart;
    if( doubleSlash.location == NSNotFound) {
        hostPortStart = 0;
    } else {
        hostPortStart = doubleSlash.location+2;
    }
    
    NSString* subStr = [url substringFromIndex:hostPortStart];
    NSRange hostPostEndSlash = [subStr rangeOfString:@"/"];
    NSInteger hostPortLength;
    if (hostPostEndSlash.location == NSNotFound) {
        hostPortLength = [url length] - hostPortStart;
    } else {
        hostPortLength = hostPostEndSlash.location;
    }
    
    NSRange portColon = [subStr rangeOfString:@":"];
    NSInteger hostLength;
    if( portColon.location == NSNotFound) {
        hostLength = hostPortLength;
    } else {
        hostLength = portColon.location;
    }
    
    NSRange range = NSMakeRange(hostPortStart, hostLength);
    host = [url substringWithRange:range];
    
    return host;
}

#pragma mark - Invoke SSL Query Dialog

/*
 * This method displays a dialog allowing the user allow relaxation of certificate verification
 */
- (void)displaySSLQueryDialog
{
    // create an alert view to query certificate verification
    UIAlertView* dialog = [[UIAlertView alloc] initWithTitle:@"SSl Certificate Error"
                                                     message:@"Relax certificate verification for this site ?\n"
                                                    delegate:self
                                           cancelButtonTitle:@"No" // index 0
                                           otherButtonTitles:@"Yes", nil];
    dialog.tag = kSSLDialog;
    
    [dialog show];
#if !(__has_feature(objc_arc))
    [dialog release];
#endif
}

#pragma mark - Invoke Auth Query Dialog
/*
 * This method displays a dialog allowing the user to enter a user id and password
 */
- (void)displayAuthQueryDialog
{
    // create an alert view with our user and password fields
    UIAlertView* dialog = [[UIAlertView alloc] initWithTitle:@"Enter ID and Password"
                                                     message:@""
                                                    delegate:self
                                           cancelButtonTitle:@"Cancel" // index 0
                                           otherButtonTitles:@"OK", nil];
    
    dialog.alertViewStyle = UIAlertViewStyleLoginAndPasswordInput;
    dialog.tag = kAuthDialog;
    
    [dialog show];
#if !(__has_feature(objc_arc))
    [dialog release];
#endif
}

#pragma mark - UIAlertViewDelegate

/*
 * Method from UIAlertViewDelegate that is actioned when a button is clicked.
 * Here we are using it to give the user a chance to relax SSL certificate verification
 *
 */
- (void)alertView:(UIAlertView*)alert clickedButtonAtIndex:(NSInteger)buttonIndex
{
    switch (alert.tag)
    {
        case kSSLDialog:
            [self handleSSLQuery:buttonIndex];
            break;
            
        case kAuthDialog:
            userNameField = [alert textFieldAtIndex:0];
            passwordField = [alert textFieldAtIndex:1];
            [self handleAuthQuery:buttonIndex];
            
            break;
            
        case kErrorAlertDialog:
            [delegate downloadDone:nil];
            break;
            
        default:
            NSLog(@"Unknown query dialog");
            break;
    }
}

#pragma mark - Query Handlers

/*
 * This method handles the response from the auth query and furnishes the supplied credentials
 * to the challenge sender
 */

- (void)handleAuthQuery:(NSInteger)buttonIndex
{
    if (buttonIndex == 1)
    {
        // setup the credentials using session cache (in memory)
        NSURLCredential* cred = [NSURLCredential credentialWithUser:userNameField.text
                                                           password:passwordField.text
                                                        persistence:NSURLCredentialPersistenceForSession];
        [[authChallenge sender] useCredential:cred forAuthenticationChallenge:authChallenge];
    }
    else
    {
        // We don't have any credentials so continue without
        [[authChallenge sender] continueWithoutCredentialForAuthenticationChallenge:authChallenge];
    }
#if !(__has_feature(objc_arc))
    [authChallenge release];
#endif
    authChallenge = nil;
}

/*
 * This method handles the response from the SSL query and reissues a request with relaxed verification
 * or fails in response to a cancel
 */
- (void)handleSSLQuery:(NSInteger)buttonIndex
{
    if (buttonIndex == 1)
    {
        // set the url
        relaxCurrentSSL = YES;
        [self requestData:currentURL relaxSSL:relaxCurrentSSL];
    }
}

#pragma mark - NSURLSession delegate methods

// we have data so add to the data buffer
- (void)URLSession:(NSURLSession *)session
          dataTask:(NSURLSessionDataTask *)dataTask
    didReceiveData:(NSData *)data {
    // Sample test code invoke GD_getFqdn API
    NSString* hostName = [self decomposeUrl:currentURL];
    char* hostFqdn = GD_getFqdn([hostName UTF8String]);
    if(hostFqdn != NULL) {
        NSString *fqdn = [[NSString alloc] initWithUTF8String:hostFqdn];
        NSLog(@"Connected to the host: %@ with FQDN: %@", hostName, fqdn);
        free(hostFqdn);
    }
    // End of test code
    
    [dataBuffer appendData:data];
}

- (void)URLSession:(NSURLSession *)session
              task:(NSURLSessionTask *)task
didCompleteWithError:(NSError *)error {
    dispatch_async(dispatch_get_main_queue(), ^{
        if (error) {
            NSInteger code = [error code];
            if (NSURLErrorServerCertificateUntrusted == code)
            {
                [self displaySSLQueryDialog];
            }
            else
            {
                // display a dialog
                NSString* str = [error localizedDescription];
                UIAlertView* dialog = [[UIAlertView alloc] initWithTitle:@"Error Fetching File"
                                                                 message:str
                                                                delegate:self
                                                       cancelButtonTitle:@"OK"
                                                       otherButtonTitles:nil];
                dialog.tag = kErrorAlertDialog;
                [dialog show];
#if !(__has_feature(objc_arc))
                [dialog release];
#endif
                
            }
        } else {
            [delegate downloadDone:dataBuffer];
            
            // reset the buffer
            [dataBuffer setLength:0];
            
            // set currentSession to nil to avoid problems if aborting after the session has gone away
            currentSession = nil;
        }
    });
}

@end
