/*
 *  This file contains Good Sample Code subject to the Good Dynamics SDK Terms and Conditions.
 *  (c) 2013 Good Technology Corporation. All rights reserved.
 */

#import <GD/GDFileSystem.h>
#import <GD/NSMutableURLRequest+GDNET.h>
#import <GD/GD_C_NETUtility.h>
#import "FeedDownloadNSUrl.h"

#define kAuthDialog 1
#define kSSLDialog  2
#define kErrorAlertDialog 101

@interface FeedDownloadHttpNSURL ()
- (void)requestData:(NSString*)url relaxSSL:(BOOL)relaxSSL;
- (void)displaySSLQueryDialog;
- (void)displayAuthQueryDialog;
- (void)handleAuthQuery:(NSInteger)buttonIndex;
- (void)handleSSLQuery:(NSInteger)buttonIndex;
- (NSString*) decomposeUrl:(NSString *) url;

@property (nonatomic, assign) BOOL isRelaxSSL;
@end

@implementation FeedDownloadHttpNSURL


- (id)init
{
    self = [super init];
    if (self)
    {
        dataBuffer = [[NSMutableData alloc] init];
    }
    return self;
}

#if !(__has_feature(objc_arc))
- (void)dealloc
{
    [challenge release];
    [dataBuffer release];
    [super dealloc];
}
#endif

#pragma mark - Request Control

/* Requests the data from a specific URL. This can trigger SSL relaxation or authentication dialogs
 */
- (void)requestData:(NSString*)url
{
    relaxCurrentSSL = NO;
    currentURL = url;
    
    [self requestData:url relaxSSL:relaxCurrentSSL];
}

/* Internal request method that can take auth parameters and relax the SSL if required
 */
- (void)requestData:(NSString*)url relaxSSL:(BOOL)relaxSSL
{
    [UIApplication sharedApplication].networkActivityIndicatorVisible = YES;
    
    NSURL* nsUrl = [NSURL URLWithString:url];
    
    // create the url/request
    NSMutableURLRequest* request = [NSMutableURLRequest requestWithURL:nsUrl];
    
    //    Optionally, to use an HTTP Proxy, uncomment the code below...
    //    NSURL* proxyUrl = [NSURL URLWithString:@"http://<hostname>:<port>"];
    //    NSURLCredential* proxyCred = [NSURLCredential credentialWithUser:@"username"
    //                                                            password:@"password"
    //                                                         persistence:NSURLCredentialPersistenceForSession];
    //
    //    NSURLProtectionSpace* proxyProtSpace = [[[NSURLProtectionSpace alloc]
    //                                             initWithProxyHost:[proxyUrl host]
    //                                                          port:[[proxyUrl port] integerValue]
    //                                                          type:[proxyUrl scheme]
    //                                                         realm:nil
    //                                          authenticationMethod:NSURLAuthenticationMethodHTTPDigest] autorelease];
    //    [request setAuthorizationCredentials:proxyCred withProtectionSpace:proxyProtSpace];
    
    self.isRelaxSSL = relaxSSL;
    
    // reset the data buffer for the new request
    [dataBuffer setLength:0];
    
    // open the request
    currentConnection = [NSURLConnection connectionWithRequest:request delegate:self];
}

- (void)abortRequest
{
    // cancel the connection if still active
    [currentConnection cancel];
    
    // set to nil
    currentConnection = nil;
    
    // switch off the activity indicator here as the normal loading flow will be halted
    [UIApplication sharedApplication].networkActivityIndicatorVisible = NO;
}

#pragma mark - NSURLConnection delegate

// there is an error so check for ssl and authentication
- (void)connection:(NSURLConnection*)connection didFailWithError:(NSError*)error
{
    NSInteger code = [error code];
    if (NSURLErrorServerCertificateUntrusted == code)
    {
        [self displaySSLQueryDialog];
    }
    else
    {
        // display a dialog
        NSString* str = [error localizedDescription];
        UIAlertView* dialog = [[UIAlertView alloc] initWithTitle:@"Error Fetching File"
                                                         message:str
                                                        delegate:self
                                               cancelButtonTitle:@"OK"
                                               otherButtonTitles:nil];
        dialog.tag = kErrorAlertDialog;
        [dialog show];
#if !(__has_feature(objc_arc))
        [dialog release];
#endif
        
    }
}

// specify if application can process authenticate challanges
- (BOOL)connection:(NSURLConnection *)connection canAuthenticateAgainstProtectionSpace:(NSURLProtectionSpace *)protectionSpace
{
    return YES;
}

- (void)connection:(NSURLConnection *)connection willSendRequestForAuthenticationChallenge:(NSURLAuthenticationChallenge *)challenge
{
    NSString* authMethod = [[challenge protectionSpace] authenticationMethod];
    NSLog(@"Auth method in use: %@", authMethod);
    
    if ([authMethod isEqualToString:NSURLAuthenticationMethodHTTPBasic] ||
        [authMethod isEqualToString:NSURLAuthenticationMethodHTTPDigest] ||
        [authMethod isEqualToString:NSURLAuthenticationMethodNTLM] ||
        [authMethod isEqualToString:NSURLAuthenticationMethodNegotiate])
    {
        [self displayAuthQueryDialog];
#if !(__has_feature(objc_arc))
        authChallenge = [challange retain];
#else
        authChallenge = challenge;
#endif
    }
    else if ([authMethod isEqualToString:NSURLAuthenticationMethodClientCertificate])
    {
	    // do nothing - GD will automatically supply an appropriate client certificate
    }
    else if ([authMethod isEqualToString:NSURLAuthenticationMethodServerTrust])
    {
        // check SSL relaxation
        if (self.isRelaxSSL &&
            [challenge.protectionSpace.authenticationMethod isEqualToString:NSURLAuthenticationMethodServerTrust])
            [challenge.sender useCredential:[NSURLCredential credentialForTrust:challenge.protectionSpace.serverTrust] forAuthenticationChallenge:challenge];
        
        [challenge.sender continueWithoutCredentialForAuthenticationChallenge:challenge];
    }
    else
    {
        // this application doesn't support the specified method
        [delegate downloadDone:nil];
    }
    
    // reset the buffer
    [dataBuffer setLength:0];
}

- (NSString*) decomposeUrl:(NSString *) url
{
    NSString* host;

    NSRange doubleSlash = [url rangeOfString:@"//"];
    NSInteger hostPortStart;
    if( doubleSlash.location == NSNotFound) {
        hostPortStart = 0;
    } else {
        hostPortStart = doubleSlash.location+2;
    }
    
    NSString* subStr = [url substringFromIndex:hostPortStart];
    NSRange hostPostEndSlash = [subStr rangeOfString:@"/"];
    NSInteger hostPortLength;
    if (hostPostEndSlash.location == NSNotFound) {
        hostPortLength = [url length] - hostPortStart;
    } else {
        hostPortLength = hostPostEndSlash.location;
    }
    
    NSRange portColon = [subStr rangeOfString:@":"];
    NSInteger hostLength;
    if( portColon.location == NSNotFound) {
        hostLength = hostPortLength;
    } else {
        hostLength = portColon.location;
    }
    
    NSRange range = NSMakeRange(hostPortStart, hostLength);
    host = [url substringWithRange:range];
   
    return host;
}

// we have data so add to the data buffer
- (void)connection:(NSURLConnection*)connection didReceiveData:(NSData*)data
{
    // Sample test code invoke GD_getFqdn API
    NSString* hostName = [self decomposeUrl:currentURL];
    char* hostFqdn = GD_getFqdn([hostName UTF8String]);
    if(hostFqdn != NULL) {
        NSString *fqdn = [[NSString alloc] initWithUTF8String:hostFqdn];
        NSLog(@"Connected to the host: %@ with FQDN: %@", hostName, fqdn);
        free(hostFqdn);
    }
    // End of test code
    
    [dataBuffer appendData:data];
}

// we have finished loading the file so sedn to the delegate
- (void)connectionDidFinishLoading:(NSURLConnection*)connection
{
    [delegate downloadDone:dataBuffer];
    
    // reset the buffer
    [dataBuffer setLength:0];
    
    // set currentConnection to nil to avoid problems if aborting after the connection has gone away
    currentConnection = nil;
}

#pragma mark - Invoke SSL Query Dialog

/*
 * This method displays a dialog allowing the user allow relaxation of certificate verification
 */
- (void)displaySSLQueryDialog
{
    // create an alert view to query certificate verification
    UIAlertView* dialog = [[UIAlertView alloc] initWithTitle:@"SSl Certificate Error"
                                                     message:@"Relax certificate verification for this site ?\n"
                                                    delegate:self
                                           cancelButtonTitle:@"No" // index 0
                                           otherButtonTitles:@"Yes", nil];
    dialog.tag = kSSLDialog;
    
    [dialog show];
#if !(__has_feature(objc_arc))
    [dialog release];
#endif
}

#pragma mark - Invoke Auth Query Dialog
/*
 * This method displays a dialog allowing the user to enter a user id and password
 */
- (void)displayAuthQueryDialog
{
    // create an alert view with our user and password fields
    UIAlertView* dialog = [[UIAlertView alloc] initWithTitle:@"Enter ID and Password"
                                                     message:@""
                                                    delegate:self
                                           cancelButtonTitle:@"Cancel" // index 0
                                           otherButtonTitles:@"OK", nil];
    
    dialog.alertViewStyle = UIAlertViewStyleLoginAndPasswordInput;
    dialog.tag = kAuthDialog;
    
    [dialog show];
#if !(__has_feature(objc_arc))
    [dialog release];
#endif
}

#pragma mark - UIAlertViewDelegate

/*
 * Method from UIAlertViewDelegate that is actioned when a button is clicked.
 * Here we are using it to give the user a chance to relax SSL certificate verification
 *
 */
- (void)alertView:(UIAlertView*)alert clickedButtonAtIndex:(NSInteger)buttonIndex
{
    switch (alert.tag)
    {
        case kSSLDialog:
            [self handleSSLQuery:buttonIndex];
            break;
            
        case kAuthDialog:
            userNameField = [alert textFieldAtIndex:0];
            passwordField = [alert textFieldAtIndex:1];
            [self handleAuthQuery:buttonIndex];
            
            break;
            
        case kErrorAlertDialog:
            /* In some cases this call crashes ( ref: https://jira.bbqnx.net/browse/GD-25075 )
             
             Crash log:
             0   libobjc.A.dylib               	0x0000000190a74150 objc_msgSend + 16
             1   RSSReader                     	0x00000001000bb4ec -[FeedDownloadHttpNSURL alertView:clickedButtonAtIndex:] (FeedDownloadNSUrl.m:310)
             2   UIKit                         	0x00000001985dd2a0 -[UIAlertView _prepareToDismissForTappedIndex:] + 152
             
             */
            if (delegate) {
                [delegate downloadDone:nil];
            } else {
                //** Remove activity indicator
                [UIApplication sharedApplication].networkActivityIndicatorVisible = NO;
            }
            
            break;
            
        default:
            NSLog(@"Unknown query dialog");
            break;
    }
}

#pragma mark - Query Handlers

/*
 * This method handles the response from the auth query and furnishes the supplied credentials
 * to the challenge sender
 */

- (void)handleAuthQuery:(NSInteger)buttonIndex
{
    if (buttonIndex == 1)
    {
        // setup the credentials using session cache (in memory)
        NSURLCredential* cred = [NSURLCredential credentialWithUser:userNameField.text
                                                           password:passwordField.text
                                                        persistence:NSURLCredentialPersistenceForSession];
        [[authChallenge sender] useCredential:cred forAuthenticationChallenge:authChallenge];
    }
    else
    {
        // We don't have any credentials so continue without
        [[authChallenge sender] continueWithoutCredentialForAuthenticationChallenge:authChallenge];
    }
#if !(__has_feature(objc_arc))
    [authChallenge release];
#endif
    authChallenge = nil;
}

/*
 * This method handles the response from the SSL query and reissues a request with relaxed verification
 * or fails in response to a cancel
 */
- (void)handleSSLQuery:(NSInteger)buttonIndex
{
    if (buttonIndex == 1)
    {
        // set the url
        relaxCurrentSSL = YES;
        [self requestData:currentURL relaxSSL:relaxCurrentSSL];
    }
}

@end
