/*
 *  This file contains Good Sample Code subject to the Good Dynamics SDK Terms and Conditions.
 *  (c) 2013 Good Technology Corporation. All rights reserved.
 */

#import "RSSManager.h"
#import <GD/GDFileManager.h>

@interface RSSManager(private)

- (void)initaliseFeedsArray;
- (void)saveFeeds;

@end

static RSSManager *sharedRSSManager = nil;

@implementation RSSManager

//Private
#define kFeedSaveFile @"feedSaveFile.dat"
NSMutableArray *rssFeeds;
RequestMode currentRequestMode;

#pragma mark - singleton method
+(RSSManager*)sharedRSSManager
{
    @synchronized(self)
    {
        if(sharedRSSManager == nil)
        {
            sharedRSSManager = [[super allocWithZone:NULL] init];
        }
    }
    return sharedRSSManager;
}


-(id)init{
    
    self = [super init];
    
    if(self)
    {
        //Default to NSURL request mode
        currentRequestMode = RequestModeNSURL;
        
        //Load the feeds to the local array
        [self initaliseFeedsArray];
        
    }
    
    return self;
}

#pragma mark - secure store and feed list

/*
 * Loads the feed table from secure storage
 */
- (void)initaliseFeedsArray
{
    NSData *dataPtr = [[GDFileManager defaultManager] contentsAtPath:[self fullPathOfFileWithName:kFeedSaveFile]];
    
    if ([dataPtr length] > 0)
    {
        rssFeeds = (NSMutableArray*)[NSKeyedUnarchiver unarchiveObjectWithData:dataPtr];
    }
    else
    {
        rssFeeds = [[NSMutableArray alloc] init];
        
        //Add Good Technology Feed as an example
        RSSFeed *goodFeed = [[RSSFeed alloc] initWithName:kRSSTitle andURL:kRSSUrl];
        [rssFeeds addObject:goodFeed];
    }
}

/*
 * Validates the RSS for name and URL and then saves locally and to secure storage
 */

-(BOOL)saveFeed:(RSSFeed*)rssFeed withName:(NSString*)feedName andURLString:(NSString*)urlString
{
    
    if(urlString.length > 0 && feedName.length > 0)
    {
        NSMutableString* urlStringAltered = [NSMutableString stringWithString:urlString];
        
        //Check for protocol
        if([urlStringAltered rangeOfString:@"://"].location == NSNotFound)
        {
            [urlStringAltered insertString:@"http://" atIndex:0];
        }
        
        //Test URL is valid
        NSURL *rssURL = [NSURL URLWithString:[NSString stringWithString:urlStringAltered]];
        
        if(rssURL)
        {
            if(rssFeed)
            {
                //update existing
                [rssFeed setRssName:feedName];
                [rssFeed setRssUrl:rssURL];
            }
            else
            {
                //Add the feed to the url
                RSSFeed *rssFeed = [[RSSFeed alloc] initWithName:feedName andURL:rssURL];
                [rssFeeds addObject:rssFeed];
            }
            
            //Save to secure storage
            [self saveFeeds];
            
            //Tell listeners there's been a successful addition
            [[NSNotificationCenter defaultCenter] postNotificationName:kRSSFeedAddedNotification object:nil];
        }
        else
        {
            return NO;
        }
        
        //inform direct caller that the feed was added successfully
        return YES;
    }
    
    return NO;
}


/*
 * Removes a feed from the local array and saves the changes to secure storage
 */
-(void)removeFeedAtPosition:(NSInteger)pos{
    
    if(rssFeeds.count > pos)
    {
        //Remove from local array
        [rssFeeds removeObjectAtIndex:pos];
        
        //Update secure storage
        [self saveFeeds];
    }
    
}


/*
 * Saves the feed table to secure storage
 */
-(void)saveFeeds
{
    //Convert RSSFeeds dictionaries
    
    NSData* encodedData = [NSKeyedArchiver archivedDataWithRootObject:rssFeeds];
    
    if(![[GDFileManager defaultManager] createFileAtPath:[self fullPathOfFileWithName:kFeedSaveFile]
                                            contents:encodedData
                                          attributes:nil])
    {
        UIAlertView* dialog = [[UIAlertView alloc] initWithTitle:@"Error Saving Feeds"
                                                         message:@"\n"
                                                        delegate:nil
                                               cancelButtonTitle:@"OK" // index 0
                                               otherButtonTitles:nil];
        dialog.tag = 0;
        [dialog show];
    }
}

#pragma mark - Request Mode
-(void)changeCurrentRequestMode:(RequestMode)_requestMode{
    
    if(currentRequestMode != _requestMode)
    {
        currentRequestMode = _requestMode;
        //Tell listeners about the change
        [[NSNotificationCenter defaultCenter] postNotificationName:kRSSRequestModeChangedNotification object:[NSNumber numberWithInt:currentRequestMode]];
    }
}


#pragma mark - helpers
//Helpers - the array is private so only this controller can directly manipulate it
-(int)numberOfFeeds{
    return (int)rssFeeds.count;
}

-(RSSFeed*)feedAtPos:(NSInteger)pos{
    return (RSSFeed*)[rssFeeds objectAtIndex:pos];
}

-(RequestMode)currentRequestMode{
    return currentRequestMode;
}

-(NSString*)fullPathOfFileWithName:(NSString*)fileName {
    
    NSArray* libPaths = NSSearchPathForDirectoriesInDomains(NSLibraryDirectory, NSUserDomainMask, YES);
    NSString* libPath = [libPaths objectAtIndex:0];
    NSString* applicationDirectory = [libPath stringByDeletingLastPathComponent];
    return [[applicationDirectory stringByAppendingString:@"/"] stringByAppendingString:fileName];
}

@end
