/*
 * (c) 2016 BlackBerry Limited. All rights reserved.
 *
 */

#import "FeedViewController.h"
#import <GD/GDFileSystem.h>
#import <GD/NSMutableURLRequest+GDNET.h>
#import "UIToolbarClear.h"
#import "UIAccessibilityIdentifiers.h"
#import <GD/GDReachability.h>
#import "Utilities.h"
#import "FeedParser.h"
#import "RSSManager.h"
#import "FeedDownloadGDHttp.h"
#import "FeedDownloadNSUrl.h"
#import "FeedDownloadNSUrlSession.h"


@interface FeedViewController () <FeedParserDelegate, UITableViewDelegate, FeedDownloadDelegate>

@property (strong, nonatomic) IBOutlet UITableView *rssStoriesTable;
@property (strong, nonatomic) NSArray *storyArray; // variables to store xml parsing elements
@property (strong, nonatomic) FeedDownloadBase *feedDownloader;
@property (strong, nonatomic) NSMutableString *currentURL; // http request
@property (strong, nonatomic) FeedParser *parser;

@end

@implementation FeedViewController

- (void)viewDidLoad
{
    [super viewDidLoad];
    
    self.rssStoriesTable.rowHeight = UITableViewAutomaticDimension;
    self.rssStoriesTable.estimatedRowHeight = 90.f;
    
    [self setTitle:self.feedInfo.rssName];
    
    //Set the url
    self.currentURL = [[NSMutableString alloc] initWithString:[self.feedInfo.rssUrl absoluteString]];
    
    if(self.feedDownloader)
    {
        [self resetFeedDownloader];
    }
    else
    {
        //See if the feed request type changes whilst stories are showing
        //Although not used in the is example - is good to know
        [[NSNotificationCenter defaultCenter] addObserver:self
                                                 selector:@selector(feedModeChanged:)
                                                     name:kRSSRequestModeChangedNotification
                                                   object:nil];
        //Create downloader
        RequestMode currentMode = [RSSManager sharedRSSManager].currentRequestMode;
        if(currentMode == RequestModeNSURL)
        {
            self.feedDownloader = [[FeedDownloadHttpNSURL alloc] init];
        }
        else if (currentMode == RequestModeGDHttp)
        {
            self.feedDownloader = [[FeedDownloadHttpGDHttp alloc] init];
        }
        else if (currentMode == RequestModeNSURLSession)
        {
            self.feedDownloader = [[FeedDownloadNSURLSession alloc] init];
        }
        
        self.feedDownloader.delegate = self;
        
        self.navigationItem.rightBarButtonItems = [self getRightBarButtonItems];
        [self refresh];
    }
    self.view.accessibilityIdentifier = BBDRssListViewID;
}

-(void)viewWillDisappear:(BOOL)animated{
    
    [super viewWillDisappear:animated];
    
    [self cleanup];
}

-(NSArray *)getRightBarButtonItems
{
    // create the array to hold the buttons, which then gets added to the toolbar
    NSMutableArray* buttons = [[NSMutableArray alloc] init];
    
    // create the stop and refresh buttons
    UIBarButtonItem* bi = [[UIBarButtonItem alloc]
                           initWithBarButtonSystemItem:UIBarButtonSystemItemStop target:self action:@selector(cancel)];
    [bi setStyle:UIBarButtonItemStylePlain];
    [buttons addObject:bi];
    
    if(![self.navigationItem respondsToSelector:@selector(setRightBarButtonItems:)])
    {
        // create a spacer
        bi = [[UIBarButtonItem alloc]
              initWithBarButtonSystemItem:UIBarButtonSystemItemFixedSpace target:nil action:nil];
        [buttons addObject:bi];
    }
    
    // create a standard "refresh" button
    bi = [[UIBarButtonItem alloc]
          initWithBarButtonSystemItem:UIBarButtonSystemItemRefresh target:self action:@selector(refresh)];
    [bi setStyle:UIBarButtonItemStylePlain];
    [buttons addObject:bi];
    
    return [NSArray arrayWithArray:buttons];
}

- (void)setStoryArray:(NSArray *)storyArray
{
    _storyArray = storyArray;
    [self.rssStoriesTable reloadData];
}

-(void)cleanup
{
    [self cancel];
    [self.feedDownloader setDelegate:nil];
}

/*
 * This method reloads the feed file and redisplays the data
 */
- (IBAction)refresh
{
    GDReachabilityStatus flags = [GDReachability sharedInstance].status;
    NSString *strMsg = [NSString stringWithFormat:@"GDReachabilityNotReachable = %@, GDReachabilityViaWiFi = %@, GDReachabilityViaCellular = %@",
                        flags == GDReachabilityNotReachable ? @"true" : @"false",
                        flags == GDReachabilityViaWiFi ? @"true" : @"false",
                        flags == GDReachabilityViaCellular ? @"true" : @"false"];
    NSLog(@"Reachability state: %@", strMsg);
  
    self.storyArray = nil;
    [self parseDataFromSite:self.currentURL];
}

/*
 * This method cancels the load by calling abort
 */
- (IBAction)cancel
{
    [self.feedDownloader abortRequest];
}

/*
 * This method reloads the feed file and redisplays the data
 */
- (void)loadFeed:(NSString*)url title:(NSString*)title
{
    
    // check if we are loading
    if (![UIApplication sharedApplication].networkActivityIndicatorVisible)
    {
        // not loading so update the table
        [self.navigationItem setTitle:title];
        
        if (!self.currentURL)
        {
            self.currentURL = [[NSMutableString alloc] initWithString:url];
        }
        else
        {
            [self.currentURL setString:url];
        }
        self.storyArray = nil;
        [self parseDataFromSite:url];
    }
    else
    {
        // pop alert view to say we are loading so please wait
        UIAlertView* dialog = [[UIAlertView alloc] initWithTitle:@"Load in Progress"
                                                         message:@"Please Wait"
                                                        delegate:self
                                               cancelButtonTitle:@"OK" // index 0
                                               otherButtonTitles:nil];
        [dialog show];
    }
}


#pragma mark - Table view data source

/*
 * returns the required number of cells for the table
 */
- (NSInteger)tableView:(UITableView*)tableView numberOfRowsInSection:(NSInteger)section
{
    return [self.storyArray count];
}

/*
 * Called for each cell when the table loads. We use it to load each cell with a story from the array.
 */
- (UITableViewCell*)tableView:(UITableView*)tableView cellForRowAtIndexPath:(NSIndexPath*)indexPath
{
    static NSString* CellIdentifier = @"Cell";
    
    UITableViewCell* cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
    // Set up the cell
    NSInteger index = indexPath.row;
    // check the dictionary containing our parsed elements
    NSString* title = self.storyArray[index][@"title"];
    // trim here as we may have some redundant preceding CR, LF or whitespaces
    NSString* desc = [self.storyArray[index][@"description"] stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceAndNewlineCharacterSet]];
    
    // add the title and description
    cell.textLabel.text = title;
    cell.detailTextLabel.text = desc;
    return cell;
}

#pragma mark - Table view delegate

- (void)tableView:(UITableView*)tableView didSelectRowAtIndexPath:(NSIndexPath*)indexPath
{
    // Does nothing in this example when an item is selected.
    [tableView deselectRowAtIndexPath:indexPath animated:YES];
}

#pragma mark - set the request API class to use

-(void)feedModeChanged:(NSNotification*)notification{
    RequestMode newRequestMode = [notification.object intValue];
    [self changeFeedRequestMode:newRequestMode forceChange:NO];
}

-(void)resetFeedDownloader
{
    RequestMode currentRequestMode = [RSSManager sharedRSSManager].currentRequestMode;
    [self changeFeedRequestMode:currentRequestMode forceChange:YES];
}

-(void)changeFeedRequestMode:(RequestMode)newRequestMode forceChange:(BOOL)isForced{
    
    switch (newRequestMode)
    {
        case RequestModeNSURL:
            // check to see if already using this mode
            if (![self.feedDownloader isMemberOfClass:[FeedDownloadHttpNSURL class]] || isForced)
            {
                // release previous
                [self.feedDownloader abortRequest];
                self.feedDownloader = [[FeedDownloadHttpNSURL alloc] init];
                self.feedDownloader.delegate = self;
            }
            break;
            
        case RequestModeGDHttp:
            // check to see if already using this mode
            if (![self.feedDownloader isMemberOfClass:[FeedDownloadHttpGDHttp class]] || isForced)
            {
                // release previous
                [self.feedDownloader abortRequest];
                self.feedDownloader = [[FeedDownloadHttpGDHttp alloc] init];
                self.feedDownloader.delegate = self;
            }
            break;
            
        case RequestModeNSURLSession:
            // check to see if already using this mode
            if (![self.feedDownloader isMemberOfClass:[FeedDownloadNSURLSession class]] || isForced)
            {
                // release previous
                [self.feedDownloader abortRequest];
                self.feedDownloader = [[FeedDownloadNSURLSession alloc] init];
                self.feedDownloader.delegate = self;
            }
            break;
            
        default:
            NSLog(@"unknown request mode - ignoring");
            break;
    }
    //Refresh the table
    [self refresh];
}


#pragma mark - Parse the feed using the chosen request API to fetch the data

/*
 * This method requests the feed file from the supplied URL
 */
- (void)parseDataFromSite:(NSString*)url
{
    [UIApplication sharedApplication].networkActivityIndicatorVisible = YES;
    [self.feedDownloader requestData:url];
}

#pragma mark - FeedDownloadDelegate Callback

/*
 * Called when the download has completed and is ready to be parsed.
 */
- (void)downloadDone:(NSData*)data
{
    [UIApplication sharedApplication].networkActivityIndicatorVisible = NO;
    
    if (data && ([data length] > 0))
    {
        FeedParser *parser = [[FeedParser alloc] initWithDelegate:self parsingData:data];
        [parser parse];
        self.parser = parser;
    }
}

- (void)didFinishParsingFeed:(NSArray *)feed
{
    self.storyArray = feed;
}

- (void)didFailWithError:(NSError *)error
{
    UIAlertController *alert = [UIAlertController alertControllerWithTitle:@"Error Parsing File"
                                                                   message:[error description]
                                                            preferredStyle:UIAlertControllerStyleAlert];
    UIAlertAction *ok = [UIAlertAction actionWithTitle:@"OK"
                                                 style:UIAlertActionStyleDefault
                                               handler:nil];
    [alert addAction:ok];
    [self presentViewController:alert animated:YES completion:nil];
}

@end
